import Router from "koa-router";
import { v4 as uuidv4 } from "uuid";
import sharp from "sharp";

import { createAvatar, Style } from "@dicebear/core";
import * as openPeeps from "@dicebear/open-peeps";
import * as pixelArt from "@dicebear/pixel-art";
import * as thumbs from "@dicebear/thumbs";

import { isValue, isSize, isStyle } from "../utils/type-guards";

const DENSITY = 72;
const avatars: Record<string, Style<any> & { defaultSize: number }> = {
  "open-peeps": { ...openPeeps, defaultSize: 704 },
  "pixel-art": { ...pixelArt, defaultSize: 16 },
  thumbs: { ...thumbs, defaultSize: 100 },
};

const validStyles = Object.keys(avatars);
const isValidStyle = isStyle(validStyles);

const router = new Router();

router.get("/list", async (ctx) => {
  const styles = validStyles.map((key) => ({ ...avatars[key].meta, key }));
  ctx.body = { styles };
});

router.use(async (ctx, next) => {
  ctx.set({
    Expires: new Date(Date.now() + 604800000).toUTCString(),
  });
  ctx.type = "image/png";
  await next();
});

router.get("/random", async (ctx) => {
  const { style = "thumbs", size = "100" } = ctx.query;
  if (!isValidStyle(style) || !isSize(size)) {
    throw Error("Invalid query");
  }
  const seed = uuidv4();
  const density = (DENSITY * parseInt(size)) / avatars[style].defaultSize;
  ctx.body = sharp(
    Buffer.from(createAvatar(avatars[style], { seed }).toString()),
    { density }
  ).png();
});

router.get("/:seed", async (ctx) => {
  const { seed } = ctx.params;
  const { style = "thumbs", size = "100" } = ctx.query;
  if (!isValue(seed) || !isValidStyle(style) || !isSize(size)) {
    throw Error("Invalid query");
  }
  const density = (DENSITY * parseInt(size)) / avatars[style].defaultSize;
  ctx.body = sharp(
    Buffer.from(createAvatar(avatars[style], { seed }).toString()),
    {
      density,
    }
  ).png();
});

export default router;
